
from setuptools import setup, find_packages

setup(
    name='pyeelight',
    version='0.4.0',
    license='MIT',
    author="Mohamed El Yousfi",
    author_email='pip@mootje.be',
    packages=find_packages(),
    url='https://github.com/Mohagames205/pyeelight-api',
    keywords=["yeelight", "api", "smart home"],
)